@@ -53,19 +53,40 @@ class BertConfig(object):
 
 class BertLayerNorm(nn.Module):
     """LayerNorm层, 见Transformer(一), 讲编码器(encoder)的第3部分"""
-    def __init__(self, hidden_size, eps=1e-12):
+    def __init__(self, hidden_size, eps=1e-12, conditional=False):
         """Construct a layernorm module in the TF style (epsilon inside the square root).
         """
         super(BertLayerNorm, self).__init__()
         self.gamma = nn.Parameter(torch.ones(hidden_size))
         self.beta = nn.Parameter(torch.zeros(hidden_size))
         self.variance_epsilon = eps
+        self.conditional = conditional
+        if conditional == True:
+            #说明是条件 ln
+            self.weight_dense = nn.Linear(2 * hidden_size, hidden_size, bias=False)
+            self.weight_dense.weight.data.uniform_(0, 0)
+            self.bias_dense = nn.Linear(2 * hidden_size, hidden_size, bias=False)
+            self.bias_dense.weight.data.uniform_(0, 0)
 
     def forward(self, x):
-        u = x.mean(-1, keepdim=True)
-        s = (x - u).pow(2).mean(-1, keepdim=True)
-        x = (x - u) / torch.sqrt(s + self.variance_epsilon)
-        return self.gamma * x + self.beta
+        if self.conditional == False:
+            u = x.mean(-1, keepdim=True)
+            s = (x - u).pow(2).mean(-1, keepdim=True)
+            x = (x - u) / torch.sqrt(s + self.variance_epsilon)
+            return self.gamma * x + self.beta
+        else :
+            inputs = x[0]
+            cond = x[1]
+            for _ in range(len(inputs.shape) - len(cond.shape)):
+                cond = cond.unsqueeze(dim=1)
+           
+            weight = self.gamma + self.weight_dense(cond)
+            bias = self.beta + self.bias_dense(cond)
+            u = inputs.mean(-1, keepdim=True)
+            s = (inputs - u).pow(2).mean(-1, keepdim=True)
+            x = (inputs - u) / torch.sqrt(s + self.variance_epsilon)
+           
+            return weight * x + bias
 
 
 
@@ -160,7 +181,7 @@ class BertSelfAttention(nn.Module):
         # seem a bit unusual, but is taken from the original Transformer paper.
         attention_probs = self.dropout(attention_probs)
 
-        # 注意力加权
+        # 注意力加权 torch.dot()
         context_layer = torch.matmul(attention_probs, value_layer)
         # 把加权后的V reshape, 得到[batch_size, length, embedding_dimension]
         context_layer = context_layer.permute(0, 2, 1, 3).contiguous()
@@ -209,7 +230,7 @@ class BertIntermediate(nn.Module):
     def __init__(self, config):
         super().__init__()
         self.dense = nn.Linear(config.hidden_size, config.intermediate_size)
-        self.intermediate_act_fn = ACT2FN[config.hidden_act]
+        self.intermediate_act_fn = ACT2FN[config.hidden_act] ## relu 
 
     def forward(self, hidden_states):
         hidden_states = self.dense(hidden_states)
